local ADDON, Package = ...
local Container = {}

function Container:refresh(bag_id)
  for bag_at = 1, NUM_CONTAINER_FRAMES do
    local bag = _G["ContainerFrame" .. bag_at]

    if bag and bag:IsShown() and bag:GetID() == bag_id then
      for slot_at = 1, C_Container.GetContainerNumSlots(bag:GetID()) do
        local slot = _G["ContainerFrame" .. bag_at .. "Item" .. slot_at]
        local border = _G["ContainerFrame" .. bag_at .. "Item" .. slot_at .. "QualityColorBorderTexture"]

        if slot and not border then
          border = slot:CreateTexture("$parentQualityColorBorderTexture", "OVERLAY", nil, 2)
          border:SetSize(68, 68)
          border:SetPoint("CENTER")
          border:SetTexture([[Interface\Buttons\UI-ActionButton-Border]])
          border:SetBlendMode("ADD")
          border:Hide()
        end

        if slot and border then
          local item_data = C_Container.GetContainerItemInfo(bag:GetID(), slot:GetID())
          local item_quest_data = C_Container.GetContainerItemQuestInfo(bag:GetID(), slot:GetID())

          if item_data and item_data.quality > LE_ITEM_QUALITY_COMMON and not item_quest_data.isQuestItem then
            border:SetVertexColor(ITEM_QUALITY_COLORS[item_data.quality].color:GetRGBA())
            border:Show()
          else
            border:Hide()
          end
        end
      end

      return nil
    end
  end
end

function Container:run()
  local agent = CreateFrame("Frame")
  agent:RegisterEvent("BANKFRAME_OPENED")
  agent:RegisterEvent("BAG_UPDATE")
  agent:SetScript("OnEvent", function(_, event, bag_id)
    if event == "BANKFRAME_OPENED" then
      for bag_id = 0, NUM_CONTAINER_FRAMES do
        self:refresh(bag_id)
      end
    end

    if event == "BAG_UPDATE" then
      self:refresh(bag_id)
    end
  end)

  hooksecurefunc("ToggleBag", function(bag_id)
    self:refresh(bag_id)
  end)

  hooksecurefunc("ToggleAllBags", function()
    for bag_id = 0, NUM_CONTAINER_FRAMES do
      self:refresh(bag_id)
    end
  end)

  hooksecurefunc("ToggleBackpack", function()
    self:refresh(0)
  end)
end

Package.container_item_quality_color_border = function()
  Container:run()
end
