local ADDON, Package = ...
local Enum = {}

function Enum.clone(t)
  local stash = {}

  for k, v in pairs(t) do
    if type(v) == "table" then
      stash[k] = Enum.clone(v)
    else
      stash[k] = v
    end
  end

  return stash
end

function Enum.rename(t, from, to)
  local stash = {}

  for k, v in pairs(t) do
    local i = k == from and to or k

    if type(v) == "table" then
      stash[i] = Enum.rename(v, from, to)
    else
      stash[i] = v
    end
  end

  return stash
end

function Enum.merge(to, from, override)
  local stash = Enum.clone(to)

  for k, v in pairs(from) do
    if type(v) == "table" and type(stash[k]) == "table" then
      stash[k] = Enum.merge(stash[k], v, override)
    else
      if stash[k] == nil or override ~= false then
        stash[k] = v
      end
    end
  end

  return stash
end

Package.Enum = Enum
