local ADDON, Package = ...

local function paperdoll_item(is_item_quality_color_border_flagged, is_item_level_flagged)
  if not is_item_quality_color_border_flagged and not is_item_level_flagged then
    return nil
  end

  local SLOTS = {
    "Head",
    "Neck",
    "Shoulder",
    "Back",
    "Chest",
    "Shirt",
    "Tabard",
    "Wrist",
    "Hands",
    "Waist",
    "Legs",
    "Feet",
    "Finger0",
    "Finger1",
    "Trinket0",
    "Trinket1",
    "MainHand",
    "SecondaryHand",
    "Ranged",
    "Ammo"
  }

  local function process(unit_id, unit_frame_type, unit_level_text, unit_paper_doll_frame)
    local unit_weapon_mainhand = nil
    local unit_weapon_offhand = nil

    local unit_item_level = 0
    local unit_item_level_text = _G[unit_paper_doll_frame:GetName() .. "ItemLevel"]

    if is_item_level_flagged and unit_paper_doll_frame and unit_level_text and not unit_item_level_text then
      unit_item_level_text = unit_paper_doll_frame:CreateFontString("$parentItemLevel", "ARTWORK", "GameFontNormalTiny")
      unit_item_level_text:SetPoint("BOTTOM", unit_level_text)
    end

    for _, item_slot_type in pairs(SLOTS) do
      local item_frame_name = unit_frame_type .. item_slot_type .. "Slot"
      local item_frame = _G[item_frame_name]
      local item_border = _G[item_frame_name .. "Border"]
      local item_level_text = _G[item_frame_name .. "LevelText"]

      if is_item_quality_color_border_flagged and item_frame and not item_border then
        local item_border_size = item_slot_type == "Ammo" and 47 or 68

        item_border = item_frame:CreateTexture(item_frame_name .. "Border", "OVERLAY", nil, -1)
        item_border:SetSize(item_border_size, item_border_size)
        item_border:SetPoint("CENTER")
        item_border:SetTexture([[Interface\Buttons\UI-ActionButton-Border]])
        item_border:SetBlendMode("ADD")
      end

      if is_item_level_flagged and item_frame and not item_level_text and item_slot_type ~= "Ammo" then
        item_level_text = item_frame:CreateFontString(item_frame_name .. "LevelText", "ARTWORK")
        item_level_text:SetFont(STANDARD_TEXT_FONT, 12, "OUTLINE")
        item_level_text:SetShadowOffset(2, -2)
        item_level_text:SetPoint("BOTTOM", 0, 1)
      end

      local item_slot_id = GetInventorySlotInfo(item_slot_type:upper() .. "SLOT")
      local item_quality = GetInventoryItemQuality(unit_id, item_slot_id)
      local item_quality_color = ITEM_QUALITY_COLORS[item_quality]
      local item_id = GetInventoryItemID(unit_id, item_slot_id)
      local item_level = GetDetailedItemLevelInfo(item_id or 0)
      local _, _, _, item_equip_loc = GetItemInfoInstant(item_id or 0)

      if item_slot_type == "MainHand" then
        unit_weapon_mainhand = item_equip_loc
      end

      if item_slot_type == "SecondaryHand" then
        unit_weapon_offhand = item_equip_loc
      end

      if item_border then
        if item_quality and item_quality_color then
          item_border:SetVertexColor(item_quality_color.r, item_quality_color.g, item_quality_color.b)
          item_border:Show()
        else
          item_border:Hide()
        end
      end

      if item_level_text then
        if item_quality and item_quality_color and item_level then
          item_level_text:SetText(item_level)
          item_level_text:SetVertexColor(item_quality_color.r, item_quality_color.g, item_quality_color.b)
          item_level_text:Show()
        else
          item_level_text:Hide()
        end
      end

      if item_level and item_slot_type ~= "Shirt" and item_slot_type ~= "Tabard" and item_slot_type ~= "Ammo" then
        unit_item_level = unit_item_level + item_level
      end
    end

    if unit_level_text and unit_item_level_text then
      local slots = 17

      if (unit_weapon_mainhand == "INVTYPE_2HWEAPON" and not IsPlayerSpell(46917) and not unit_weapon_offhand)
      or (not unit_weapon_mainhand and not unit_weapon_offhand) then
        slots = 16
      end

      unit_level_text:Hide()
      unit_item_level_text:SetText(STAT_AVERAGE_ITEM_LEVEL .. " " .. math.floor(unit_item_level / slots) .. " " .. unit_level_text:GetText())
    end
  end

  local function OnPlayer() process("player", "Character", CharacterLevelText, PaperDollFrame) end
  local function OnTarget() process("target", "Inspect", InspectLevelText, InspectPaperDollFrame) end

  local function OnEvent(self, event)
    if event == "PLAYER_EQUIPMENT_CHANGED" then
      OnPlayer()
    elseif event == "INSPECT_READY" then
      OnTarget()
    end
  end

  LoadAddOn("Blizzard_InspectUI")

  PaperDollFrame:HookScript("OnShow", OnPlayer)
  InspectPaperDollFrame:HookScript("OnShow", OnTarget)

  local agent = CreateFrame("Frame")
  agent:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
  agent:RegisterEvent("INSPECT_READY")
  agent:SetScript("OnEvent", OnEvent)
end

local function paperdoll_name()
  local function process(unit_id, unit_name_text)
    local unit_name = UnitName(unit_id)
    local unit_class = UnitClassBase(unit_id)

    if unit_name and unit_class then
      unit_name_text:SetText(RAID_CLASS_COLORS[unit_class]:WrapTextInColorCode(unit_name))
    end
  end

  local function OnPlayer() process("player", CharacterNameText) end
  local function OnTarget() process("target", InspectNameText) end

  LoadAddOn("Blizzard_InspectUI")

  CharacterFrame:HookScript("OnShow", OnPlayer)
  InspectFrame:HookScript("OnShow", OnTarget)
end

Package.paperdoll_item = paperdoll_item
Package.paperdoll_name = paperdoll_name
