local ADDON, Package = ...

local function trade_recipient_name_with_class_and_level()
  local function TradeFrame_Update()
    local class = UnitClassBase("npc")
    local level = UnitLevel("npc")

    if class and level > 0 then
      TradeFrameRecipientNameText:SetFontObject("GameFontNormalSmall")
      TradeFrameRecipientNameText:SetFormattedText(
        "%s%s|r %s%s|r",
        ConvertRGBtoColorString(GetQuestDifficultyColor(level)),
        level,
        ConvertRGBtoColorString(RAID_CLASS_COLORS[class]),
        TradeFrameRecipientNameText:GetText()
      )
    end
  end

  hooksecurefunc("TradeFrame_Update", TradeFrame_Update)
end

local function trade_item_quality_color_border()
  local units = { Player = GetTradePlayerItemInfo, Recipient = GetTradeTargetItemInfo }

  for unit_name, _ in pairs(units) do
    for at = 1, 7 do
      local item_frame_name = "Trade" .. unit_name .. "Item" .. at .. "ItemButton"
      local item_frame = _G[item_frame_name]
      local item_border = item_frame:CreateTexture(item_frame_name .. "Border", "OVERLAY", nil, -1)
      item_border:SetSize(68, 68)
      item_border:SetPoint("CENTER")
      item_border:SetTexture([[Interface\Buttons\UI-ActionButton-Border]])
      item_border:SetBlendMode("ADD")
    end
  end

  local function OnEvent(self, event, ...)
    for unit_name, GetTradeItemInfo in pairs(units) do
      for at = 1, 7 do
        local item_name, _, _, item_quality = GetTradeItemInfo(at)
        local item_border = _G["Trade" .. unit_name .. "Item" .. at .. "ItemButtonBorder"]
        local item_quality_color = ITEM_QUALITY_COLORS[item_quality]

        if item_border then
          if item_name and item_quality and item_quality_color then
            item_border:SetVertexColor(item_quality_color.r, item_quality_color.g, item_quality_color.b)
            item_border:Show()
          else
            item_border:Hide()
          end
        end
      end
    end
  end

  local agent = CreateFrame("Frame")
  agent:RegisterEvent("TRADE_SHOW")
  agent:RegisterEvent("TRADE_UPDATE")
  agent:RegisterEvent("TRADE_PLAYER_ITEM_CHANGED")
  agent:RegisterEvent("TRADE_TARGET_ITEM_CHANGED")
  agent:SetScript("OnEvent", OnEvent)
end

Package.trade_recipient_name_with_class_and_level = trade_recipient_name_with_class_and_level
Package.trade_item_quality_color_border = trade_item_quality_color_border
